@@ -20,10 +20,6 @@ import yaml
 from dateutil.parser import parse
 from docker import DockerClient
 from docker.models.containers import Container
-from zuper_commons.fs import AbsDirPath, mkdirs_thread_safe, write_ustring_to_utf8_file
-from zuper_commons.text import box, indent
-from zuper_commons.timing import now_utc
-from zuper_commons.types import ZException
 
 from dt_shell import ConfigNotPresent
 from dt_shell.config import get_shell_config_default, read_shell_config
@@ -66,6 +62,10 @@ from duckietown_challenges import (
     ZValueError,
 )
 from duckietown_docker_utils.docker_run import get_developer_volumes
+from zuper_commons.fs import AbsDirPath, mkdirs_thread_safe, write_ustring_to_utf8_file
+from zuper_commons.text import box, indent
+from zuper_commons.timing import now_utc
+from zuper_commons.types import ZException
 from . import __version__, logger
 from .constants import LABEL_CREATED_BY_RUNNER, LABEL_RUNNER_NAME
 from .docker_compose import (
@@ -104,6 +104,51 @@ def dt_challenges_evaluator():
         sys.exit(2)
 
 
+#
+# def true_if_present(v: str):
+#     v2 = v.upper().replace("-", "_")
+#     ev = f"DT_RUNNER_{v2}"
+#     res = ev in os.environ
+#     if not res:
+#         logger.debug(f"not found {ev}")
+#     else:
+#         logger.info(f"Using {v=} = {ev} = {res!r}")
+#     return res
+
+
+def get_truthy_value_or_default(v: str, default: bool) -> bool:
+    v2 = v.upper().replace("-", "_")
+    ev = f"DT_RUNNER_{v2}"
+    if ev not in os.environ:
+        logger.debug(f"not found {ev} - default = {default}")
+        return default
+    else:
+        res = as_truthy(os.environ[ev])
+        logger.info(f"Using {v=} = {ev} = {res!r}")
+        return res
+
+
+def as_truthy(s: str) -> bool:
+    s = s.lower()
+    if s in ["0", "false", "no"]:
+        return False
+    if s in ["1", "true", "yes"]:
+        return True
+    raise ValueError(f"Cannot convert {s!r} to a truthy value.")
+
+
+def get_env_value_or_none(v: str, T, default_value=None):
+    v2 = v.upper().replace("-", "_")
+    ev = f"DT_RUNNER_{v2}"
+    if ev not in os.environ:
+        logger.debug(f"not found {ev}, using default {v=} = {default_value!r}")
+        return default_value
+    val = os.environ[ev]
+    logger.info(f"Using {v=} = {ev} = {val!r}")
+    res = T(val)
+    return res
+
+
 # noinspection PyBroadException
 def dt_challenges_evaluator_(
     args: List[str] = None, token: Optional[str] = None, credentials: Optional[DockerCredentials] = None
@@ -127,57 +172,39 @@ Usage:
         prog=prog, usage=usage, formatter_class=argparse.ArgumentDefaultsHelpFormatter
     )
 
-    def true_if_present(v: str):
-        v2 = v.upper().replace("-", "_")
-        ev = f"DT_RUNNER_{v2}"
-        res = ev in os.environ
-        if not res:
-            logger.debug(f"not found {ev}")
-        else:
-            logger.info(f"Using {v=} = {ev} = {res!r}")
-        return res
-
-    def get_env_value_or_none(v: str, T, default_value=None):
-        v2 = v.upper().replace("-", "_")
-        ev = f"DT_RUNNER_{v2}"
-        if ev not in os.environ:
-            logger.debug(f"not found {ev}, using default {v=} = {default_value!r}")
-            return default_value
-        val = os.environ[ev]
-        logger.info(f"Using {v=} = {ev} = {val!r}")
-        res = T(val)
-        return res
-
     parser.add_argument("--version", action="store_true", default=False, help="Shows version and exits")
     parser.add_argument(
-        "--continuous", action="store_true", default=true_if_present("continuous"), help="Runs continuously"
+        "--continuous",
+        action="store_true",
+        default=get_truthy_value_or_default("continuous", False),
+        help="Runs continuously",
     )
     parser.add_argument(
         "--no-pull",
         dest="no_pull",
         action="store_true",
-        default=true_if_present("no-pull"),
+        default=get_truthy_value_or_default("no-pull", False),
         help="Avoid pulling images",
     )
     parser.add_argument(
         "--no-upload",
         dest="no_upload",
         action="store_true",
-        default=true_if_present("no-upload"),
+        default=get_truthy_value_or_default("no-upload", False),
         help="Avoid uploading artefacts",
     )
     parser.add_argument(
         "--no-delete",
         dest="no_delete",
         action="store_true",
-        default=true_if_present("no-delete"),
+        default=get_truthy_value_or_default("no-delete", False),
         help="Avoids deleting local evaluation",
     )
     parser.add_argument(
         "--no-cache",
         dest="no_cache",
         action="store_true",
-        default=true_if_present("no-cache"),
+        default=get_truthy_value_or_default("no-cache", False),
         help="Avoiding using local cache",
     )
     parser.add_argument("--ipfs", dest="ipfs", action="store_true", default=False)
@@ -199,7 +226,7 @@ Usage:
         default=get_env_value_or_none("quota-cpu", float),
     )
     user = getpass.getuser()
-    DEFAULT_TMPDIR = f"/tmp/{user}/duckietown/DT18/evaluator/executions"
+    DEFAULT_TMPDIR = f"/tmp/{user}/duckietown-challenges-runner"
 
     parser.add_argument(
         "--tmpdir", help="Location of temp dir", default=get_env_value_or_none("tmpdir", str, DEFAULT_TMPDIR)
@@ -209,7 +236,7 @@ Usage:
         help="Credentials YAML dictionary",
         default=get_env_value_or_none("credentials", str, None),
     )
-    parser.add_argument("--submission", default=None, help="evaluate this particular submission")
+    parser.add_argument("--submission", default=None, type=int, help="evaluate this particular submission")
     parser.add_argument(
         "--reset",
         dest="reset",
@@ -234,7 +261,7 @@ Usage:
         return
 
     copy_to_machine_cache = not parsed.no_cache
-
+    logger.info(f"copy_to_machine_cache = {copy_to_machine_cache!r}")
     if parsed.credentials:
         c = parsed.credentials
         try:
@@ -277,6 +304,7 @@ Usage:
     else:
         evaluator_name = "noname" + "_" + precise_str
 
+    evaluator_name = f"{evaluator_name}-{__version__}"
     if parsed.machine_id:
         machine_id = parsed.machine_id  # + "_" + socket.gethostname()
     else:
@@ -406,16 +434,19 @@ def is_gpu_present():
     return res
 
 
-def get_features(more_features, use_ipfs: bool) -> EvaluatorFeaturesDict:
+def get_features(more_features, tmpdir: str, use_ipfs: bool) -> EvaluatorFeaturesDict:
     import psutil
 
     features = {}
 
     machine = platform.machine()
+
     features["linux"] = sys.platform.startswith("linux")
     features["mac"] = sys.platform.startswith("darwin")
     features["x86_64"] = machine == "x86_64"
-    features["armv7l"] = machine == "armv7l"
+    features["armv7l"] = "arm" in machine or machine in ["aarch64"]
+    features[machine] = True
+
     meminfo = psutil.virtual_memory()
     # svmem(total=16717422592, available=5376126976, percent=67.8, used=10359984128, free=1831890944,
     # active=7191916544, inactive=2325667840, buffers=525037568, cached=4000509952, shared=626225152)
@@ -423,15 +454,19 @@ def get_features(more_features, use_ipfs: bool) -> EvaluatorFeaturesDict:
     features["ram_total_mb"] = int(meminfo.total / (1024 * 1024.0))
     features["ram_available_mb"] = int(meminfo.available / (1024 * 1024.0))
     features["nprocessors"] = psutil.cpu_count()
-    cpu_freq = psutil.cpu_freq()
-    if cpu_freq is not None:
-        # None on Docker
-        features["processor_frequency_mhz"] = int(psutil.cpu_freq().max)
+    try:
+        cpu_freq = psutil.cpu_freq()
+    except:
+        pass
+    else:
+        if cpu_freq is not None:
+            # None on Docker
+            features["processor_frequency_mhz"] = int(psutil.cpu_freq().max)
     f = psutil.cpu_percent(interval=0.2)
     features["processor_free_percent"] = int(100.0 - f)
     features["p1"] = True
 
-    disk = psutil.disk_usage(os.getcwd())
+    disk = psutil.disk_usage(tmpdir)
 
     features["disk_total_mb"] = disk.total / (1024 * 1024)
     features["disk_available_mb"] = disk.free / (1024 * 1024)
@@ -458,6 +493,7 @@ def get_features(more_features, use_ipfs: bool) -> EvaluatorFeaturesDict:
         features[k] = v
 
     # logger.debug(json.dumps(features, indent=4))
+    logger.debug(f"Machine: {machine}", features=features)
 
     return features
 
@@ -476,6 +512,7 @@ def upload_process(
     process_id: str,
     evaluator_version: str,
     impersonate: bool,
+    copy_to_machine_cache: bool,
 ):
     from zuper_commons.logs import setup_logging
 
@@ -518,7 +555,9 @@ def upload_process(
                 logger.info(f"found {len(toupload)} artefacts to upload", existing_but_updated=updated)
 
                 try:
-                    _uploaded = upload(aws_config, toupload, quiet=True)
+                    _uploaded = upload(
+                        aws_config, toupload, copy_to_machine_cache=copy_to_machine_cache, quiet=True
+                    )
                 except:
                     raise
                 else:
@@ -582,7 +621,7 @@ def go_(
 ):
     if use_ipfs:
         allow_host_network = True
-    features = get_features(more_features, use_ipfs=use_ipfs)
+    features = get_features(more_features, tmpdir=tmpdir, use_ipfs=use_ipfs)
     # if features['processor_free_percent'] < 10:
     #     logger.info(f"Waiting because free CPU is {features['processor_free_percent']}%.")
     #     return
@@ -653,11 +692,16 @@ def go_(
     ctx = multiprocessing.get_context("spawn")
 
     submission_id = res["submission_id"]
+    step_name = res["step_name"]
     wd0 = os.path.join(
         tmpdir,
+        "evaluations",
+        machine_id,
+        evaluator_name,
         res["challenge_name"],
-        f"submission{submission_id}",
-        f"{res['step_name']}-{evaluator_name}-job{job_id}",
+        f"submission-{submission_id}",
+        f"job-{job_id}-step-{step_name}",
+        # f"{res['step_name']}-{evaluator_name}-job{job_id}",
     )
     wd = wd0 + "-a-wd"
     fd = wd0 + "-a-fifos"
@@ -672,6 +716,7 @@ def go_(
         process_id,
         evaluator_version,
         impersonate,
+        copy_to_machine_cache,
     )
     p = ctx.Process(target=upload_process, args=params, daemon=True)
     p.start()
